/*
*  Rubrica
*  file: 
*
*  
*  Copyright (C) 2000-2005 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <glade/glade.h>
#include <libral.h>

#include "types.h"
#include "app.h"
#include "adb.h"
#include "callbacks.h"
#include "cards.h"
#include "utils.h"
#include "view.h"
#include "browse.h"
#include "personal_gui.h"
#include "groups_cb.h"
#include "popmenu.h"




RubricaFilterType rrf[] = {
  {"Rubrica addressbook (*.rub)", "*.rub"},
  {"VCard addressbook (*.vcard)", "*.vcard"},
  {"CSV addressbook (*.csv)",     "*.csv"},
  {NULL, NULL}
};



void 
on_new_cb (GtkWidget* button, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  RubricaAdb* adb;
  
  adb = rubrica_adb_new();
  rubrica_app_append_addressbook(app, adb);
}


void 
on_open_cb (GtkWidget* button, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  GtkWidget* parent;
  GtkWidget* chooser;
  GtkFileFilter* filter;
  RubricaFilterType *f = rrf;

  parent = rubrica_app_get_main_window(app);
  chooser = gtk_file_chooser_dialog_new("Open", 
					(GtkWindow*) parent,
					GTK_FILE_CHOOSER_ACTION_OPEN,
					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					NULL);

  for (; f->filter_name; f++)
    {
      filter = gtk_file_filter_new();
      gtk_file_filter_set_name(filter, f->filter_name);
      gtk_file_filter_add_pattern (filter, f->filter_pattern);
      gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(chooser), filter);
    }

  if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_ACCEPT)
    {
      gchar* file;
      RubricaAdb* adb;
      RubricaAbook* abook;
      RubricaGroupSet* set;
      
      file  = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
      adb   = rubrica_adb_new();
      abook = rubrica_adb_get_addressbook(adb);

      if(rubrica_abook_open_file(RUBRICA_ABOOK(abook), file))
	{
	  g_print("\nRubrica opened successfully");
	  g_print("\nfile: %s\npath: %s",
		  rubrica_abook_get_name(RUBRICA_ABOOK(abook)),
		  rubrica_abook_get_path(RUBRICA_ABOOK(abook)));
	  g_print("\nCards in this addressbook: %d", 
		  rubrica_abook_get_items(RUBRICA_ABOOK(abook)));
	  set = rubrica_load_get_user_group();
	  
	  rubrica_adb_set_name(adb, file);
	  rubrica_adb_set_user_groups(adb, set);
	  rubrica_app_append_addressbook(app, adb);	  
	}
      else
	{
	  // TODO error message
	  rubrica_adb_free(adb);
	}
      
      g_free (file);
    }

  gtk_widget_destroy(chooser);
}


void 
confirm_overwrite (RubricaApp* app, RubricaAbook* book)
{
  GtkWidget* message;
  GtkWidget* window;
  gint response;

  window = rubrica_app_get_main_window(app);

  message = gtk_message_dialog_new(GTK_WINDOW(window), 
				   GTK_DIALOG_MODAL, 
				   GTK_MESSAGE_WARNING,
				   GTK_BUTTONS_YES_NO,
				   "%s", "Save addressbook");
  gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(message), "%s", 
					   "Addressbook exist.\n"
					   "Do you want overwrite?");
  response = gtk_dialog_run(GTK_DIALOG(message));
  switch(response)
    {
    case GTK_RESPONSE_YES:
      if (rubrica_abook_overwrite_file(book))
	g_print("\nRubrica *** File replaced successfully");	  
      break;
      
    case GTK_RESPONSE_NO:
      break;
      
    default:
      break;
    }
  
  gtk_widget_destroy(message);
}


void 
on_save_cb (GtkWidget* button, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  RubricaAbook* abook;
  RubricaAdb* adb;
  gchar* name;
  gchar* file;
  gchar* path;

  adb   = rubrica_app_get_active_addressbook(app);
  abook = rubrica_adb_get_addressbook(adb);
    
  file  = rubrica_abook_get_name(abook);
  path  = rubrica_abook_get_path(abook);
  name  = g_strdup_printf("%s%s%s", path, G_DIR_SEPARATOR_S, file);

  if (g_file_test(name, G_FILE_TEST_EXISTS))
    confirm_overwrite(app, abook);
  else
    rubrica_abook_save_file(abook, name);  
  
  g_free(file);
  g_free(path);
  g_free(name);
}


void 
on_saveas_cb (GtkWidget* button, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  RubricaAdb* adb;
  GtkWidget* parent;
  GtkWidget* chooser;
  GtkFileFilter* filter;
  RubricaFilterType* f = rrf;
  
  adb = rubrica_app_get_active_addressbook(app);
  
  parent = rubrica_app_get_main_window(app);
  chooser = gtk_file_chooser_dialog_new("Save as...", 
					(GtkWindow*) parent,
					GTK_FILE_CHOOSER_ACTION_SAVE,
					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
					NULL);
  
  for (; f->filter_name; f++)
    {
      filter = gtk_file_filter_new();
      gtk_file_filter_set_name(filter, f->filter_name);
      gtk_file_filter_add_pattern (filter, f->filter_pattern);
      gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(chooser), filter);
    }

  if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_ACCEPT)
    {
      gchar* file;
      gchar* oldfile;
      RubricaAbook* abook;
      
      abook = rubrica_adb_get_addressbook(adb);

      file    = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
      oldfile = rubrica_abook_get_name(abook);

      if (0 != rubrica_abook_get_items(abook))
	{
	  gchar* tmp;

	  if ((g_ascii_strcasecmp(file, oldfile) == 0) &&
	      g_file_test(oldfile, G_FILE_TEST_EXISTS))
	    {
	      tmp = g_strdup_printf("%s~", oldfile);

	      g_rename(oldfile, tmp);
	      g_remove(oldfile);

	      g_free(tmp);
	    }

	  g_free(oldfile);

	  if(rubrica_abook_save_file(abook, file))
	    {
	      //	      if (!do_backup()) g_remove(oldfile);
	      g_print("\n%s saved", file);
	    } 
	}
    }
  
  gtk_widget_destroy(chooser);
}



void 
on_cut_cb  (GtkWidget* button, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  RubricaAdb* adb;
  RubricaAbook* book;
  gchar *id;
  gpointer card = NULL;

  adb  = rubrica_app_get_active_addressbook(app);
  book = rubrica_adb_get_addressbook(adb);
  id   = rubrica_abook_get_selected_card_id(RUBRICA_ABOOK(book));
  card = rubrica_abook_get_card_by_id(RUBRICA_ABOOK(book), id);
  
  if (card)
    {
      rubrica_app_push(app, card);
      rubrica_app_notify(app, _("Card cutted"));

      g_object_set(RUBRICA_CARD(card), "deleted", TRUE, NULL);
      // TODO: settare a grigio la riga
    }
}


void 
on_copy_cb (GtkWidget* button, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  RubricaAdb* adb;
  RubricaAbook* book;
  gchar *id;
  gpointer card, copy = NULL;

  adb  = rubrica_app_get_active_addressbook(app);
  book = rubrica_adb_get_addressbook(adb);
  id   = rubrica_abook_get_selected_card_id(RUBRICA_ABOOK(book));
  card = rubrica_abook_get_card_by_id(RUBRICA_ABOOK(book), id);
  copy = rubrica_card_copy(RUBRICA_CARD(card));
  
  if (copy)
    {
      rubrica_app_push(app, copy);
      rubrica_app_notify(app, _("Card copied"));
    }
}


void 
on_paste_cb (GtkWidget* button, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  RubricaAdb* adb;
  RubricaAbook* book;
  gpointer card = NULL;

  adb  = rubrica_app_get_active_addressbook(app);
  book = rubrica_adb_get_addressbook(adb);
  card = rubrica_app_pop(app);
  if (card)
    {
      rubrica_abook_add_card(RUBRICA_ABOOK(book), RUBRICA_CARD(card));
      rubrica_app_notify(app, _("Card pasted"));     
    }
}



void 
on_add_personal_cb (GtkWidget* button, gpointer data)
{
  cards_append_personal_card(RUBRICA_APP(data));
}


void 
on_add_company_cb (GtkWidget* button, gpointer data)
{
  cards_append_company_card(RUBRICA_APP(data));
  //  build_personal_form(RUBRICA_ABOOK(data));
}


void 
on_delete_card_cb(GtkWidget* button, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  RubricaAdb* adb;
  RubricaAbook* book;
  gchar *id;
  gpointer card = NULL;

  adb  = rubrica_app_get_active_addressbook(app);
  book = rubrica_adb_get_addressbook(adb);
  id   = rubrica_abook_get_selected_card_id(RUBRICA_ABOOK(book));
  card = rubrica_abook_get_card_by_id(RUBRICA_ABOOK(book), id);
  
  if (card)
    {
      rubrica_app_push(app, card);
      rubrica_app_notify(app, _("Card cutted"));
      
      rubrica_abook_remove_card(RUBRICA_ABOOK(book), card);
      rubrica_app_update_cards_view_on_delete(app);
      rubrica_app_move_card_to_trash(app, card);    
    }  
}

void 
on_modify_card_cb (GtkWidget* button, gpointer data)
{ 
  RubricaApp* app = (RubricaApp*) data;

  cards_modify_personal_card(app);
}


void 
on_prefenences_cb(GtkWidget* button, gpointer data)
{
  // rubrica_preferences_run();
}


void 
on_info_cb (GtkWidget* button, gpointer data)
{
  GladeXML* gui;  
  GtkWidget* dialog;
  
  gui = glade_xml_new (RUBRICA_GUI_DIR"/main/Info.glade", NULL, NULL);
  if (!gui)
    g_error("Can't load gui");  
  
  dialog = glade_xml_get_widget(gui, "info");
  g_object_unref(gui);

  while (gtk_dialog_run (GTK_DIALOG(dialog)) != GTK_RESPONSE_CLOSE);
  
  gtk_widget_destroy(dialog);  
}



void on_print_cb(GtkWidget* button, gpointer data)
{
  // print selected card
}

void on_close_cb(GtkWidget* button, gpointer data)
{
  // close active addressbook
}



void on_first_cb(GtkWidget* button, gpointer data)
{
  rubrica_browse_first_card(RUBRICA_APP(data));
}


void on_last_cb(GtkWidget* button, gpointer data)
{
  rubrica_browse_last_card(RUBRICA_APP(data));
}



void on_prev_cb(GtkWidget* button, gpointer data)
{
  rubrica_browse_previous_card(RUBRICA_APP(data));
}


void on_next_cb (GtkWidget* button, gpointer data)
{
  rubrica_browse_next_card(RUBRICA_APP(data));
}


void on_search_cb(GtkWidget* button, gpointer data)
{
  GladeXML* gui;  
  GtkWidget* dialog;
  gint response;
  
  gui = glade_xml_new (RUBRICA_GUI_DIR"/search/Search.glade", NULL, NULL);
  if (!gui)
    g_error("Can't load gui");  
  
  dialog = glade_xml_get_widget(gui, "RubricaSearch");
  g_object_unref(gui);

  response = gtk_dialog_run (GTK_DIALOG(dialog));
  
  switch(response)
    {
    case GTK_RESPONSE_ACCEPT:
      break;

    case GTK_RESPONSE_CLOSE:
      break;
    }
  
  gtk_widget_destroy(dialog);    
}


void on_quit_cb(GtkWidget* button, gpointer data)
{
  gtk_main_quit();
}


void 
on_card_added (RubricaAbook* book, gpointer the_card, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  RubricaCard* card = (RubricaCard*) the_card;
  
  rubrica_app_add_card_to_view(app, card);
  rubrica_app_update_bookstree_view(app);
}



/**
 * on_card_removed
 * @book: RubricaAbook*
 * @card: deleted card
 * data: user data (RubricaApp*)
 *
 */
void 
on_card_removed  (RubricaAbook* book, gpointer card, gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  
  rubrica_app_update_bookstree_view(app);
}


void 
on_card_modified (RubricaAbook* book, gpointer newcard, gpointer data)
{
  RubricaApp *app = (RubricaApp*) data;
  RubricaAdb *adb = NULL;

  GtkTreeView      *view = NULL;
  GtkTreeModel     *model;
  GtkTreeSelection *selection;
  GtkTreeIter iter;
  
  adb   = rubrica_app_get_active_addressbook (app);
  view  = rubrica_adb_get_tree(adb);
  model = gtk_tree_view_get_model(view);
  selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));

  if (gtk_tree_selection_get_selected(selection, NULL, &iter))
    {
      RubricaInfos* infos;

      infos = rubrica_card_get_infos(RUBRICA_CARD(newcard));
      
      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model), &iter,
			 CARD_COLUMN,       rubrica_infos_get_label(infos), 
			 FIRST_NAME_COLUMN, rubrica_infos_get_first(infos), 
			 LAST_NAME_COLUMN,  rubrica_infos_get_last(infos),
			 PROFESSION_COLUMN,rubrica_infos_get_profession(infos),
			 CITY_COLUMN,       rubrica_infos_get_city(infos),
			 COUNTRY_COLUMN,    rubrica_infos_get_country(infos),
			 WEB_CARD_COLUMN,   rubrica_infos_get_web(infos),
			 EMAIL_CARD_COLUMN, rubrica_infos_get_email(infos),
			 TELEPHONE_COLUMN,  rubrica_infos_get_telephone(infos),
			 ASSIGMENT_COLUMN, rubrica_infos_get_assignment(infos),
			 ID_COLUMN,         rubrica_infos_get_id(infos),
			 -1);   
      
      rubrica_app_update_bookstree_view(app);
      rubrica_card_free_infos(infos);
    }
}




/**************
 */
void
on_notebook_switch_page(GtkNotebook *notebook, GtkNotebookPage *page,
			guint page_num,	gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;  
  RubricaAdb* adb = NULL;
  GtkWidget* box;

  box = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), page_num);

  if (page_num == 0)
    adb = g_object_get_data(G_OBJECT(box), "Trash");  
  else
    adb = g_object_get_data(G_OBJECT(box), "Addressbook");  
    
  rubrica_app_set_active_addressbook(app, adb);  
}




/*   events
*/
void
on_selection_changed (GtkTreeSelection *selection, gpointer data)
{
  RubricaApp *app = (RubricaApp*) data;
  GtkTreeModel *model;
  GtkTreeIter iter;
  gint page;
  
  if (gtk_tree_selection_get_selected(selection, &model, &iter))
    {
      gtk_tree_model_get(model, &iter, ADDRESSBOOK_PAGE_COLUMN, &page, -1);
      
      if (page >= 0)
	rubrica_app_show_page(app, page);
    }
}


void 
on_card_tree_event_occurred (GtkTreeView* tree, GdkEvent *event,
			     gpointer data)
{
  RubricaApp* app = (RubricaApp*) data;
  RubricaAdb* adb;
  RubricaAbook* book;
  GtkTreeModel *model;
  GtkTreeSelection *selection; 
  GtkTreePath *path = NULL;
  GtkTreeIter iter;
  GdkEventButton *ev = (GdkEventButton *)event;
  gchar* id;

  adb       = rubrica_app_get_active_addressbook (app);
  book      = rubrica_adb_get_addressbook(adb);
  model     = gtk_tree_view_get_model(GTK_TREE_VIEW(tree));
  selection = gtk_tree_view_get_selection(tree);

  gtk_tree_selection_unselect_all(selection);
  
  /* rigth button pressed  ==> show popup menu */
  if ((ev->type == GDK_BUTTON_PRESS) && (ev->button == 3))
    {
      GtkMenu* menu;  
      
      menu = (GtkMenu*) rubrica_menu_do_card_popup(RUBRICA_APP(app), 
						   tree, event);
      
      gtk_widget_show_all(GTK_WIDGET(menu));
      gtk_menu_popup (menu, NULL, NULL, NULL, NULL, ev->button, ev->time);
    }
  
  /* if a card is selected */
  if (gtk_tree_view_get_path_at_pos(tree, ev->x, ev->y, &path, 
				    NULL, NULL, NULL))
    {
      gtk_tree_selection_select_path(selection, path);
      gtk_tree_model_get_iter(model, &iter, path);
      gtk_tree_path_free(path);

      gtk_tree_model_get(model, &iter, ID_COLUMN, &id, -1);
      rubrica_abook_set_selected_card_id(RUBRICA_ABOOK(book), id);

      /* left button clicked (card selected, show it) */
      if ((ev->type == GDK_BUTTON_PRESS) && (ev->button == 1))
	{
	  gpointer card;	    
		  
	  card = rubrica_abook_get_card_by_id(RUBRICA_ABOOK(book), id);
	  
	  rubrica_view_show_data(RUBRICA_APP(app), card);
	}	
      else /* left button double click  ==> modify selected card */
	if ((ev->type == GDK_2BUTTON_PRESS) && (ev->button == 1))
	  {	           
	    cards_modify_personal_card(RUBRICA_APP(app));
	  }
    }
}




