/*
*  Rubrica
*  file: 
*
*  
*  Copyright (C) 2000-2005 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtk.h>
#include <libral.h>

#include "models.h"
#include "utils.h"


GtkTreeModel* 
rubrica_model_new(RubricaModelType type)
{
  GtkListStore *model = NULL;
  
  switch (type)
    {
      /* addressbook's cards model */
    case CARDS_MODEL:
      model = gtk_list_store_new(COLUMNS, 
				 G_TYPE_STRING,       /* card label */
				 G_TYPE_STRING,       /* first name */ 
				 G_TYPE_STRING,       /* last name  */ 
				 G_TYPE_STRING,       /* profession */  
				 G_TYPE_STRING,       /* city       */
				 G_TYPE_STRING,       /* country    */
				 G_TYPE_STRING,       /* web        */
				 G_TYPE_STRING,       /* email      */ 
				 G_TYPE_STRING,       /* telephone  */ 
				 G_TYPE_STRING,       /* company    */  
				 G_TYPE_STRING,       /* assigment  */
				 G_TYPE_STRING);      /* card's id  */
      break;
      
      /* opened addressbooks model (nb == notebook */
    case ADDRESSBOOK_MODEL:
      return (GtkTreeStore*) gtk_tree_store_new(ADDRESSBOOK_LAST_COLUMN,
						GDK_TYPE_PIXBUF, /* pixmap   */
						G_TYPE_STRING,   /* filename */
						G_TYPE_INT);     /* nb page  */
      
      break;

      /* categories model */
    case GROUP_MODEL:
      model = gtk_list_store_new(GROUP_LAST_COLUMN,
				 G_TYPE_INT,        /* group check (belong) */
				 GDK_TYPE_PIXBUF,   /* group pixbuf */
				 G_TYPE_STRING,     /* group label  */
				 G_TYPE_STRING,     /* group owner  */
				 G_TYPE_STRING);    /* pixmap path  */
   
      break;
      
      /* Address model */
    case ADDRESS_MODEL:
      model = gtk_list_store_new(LAST_ADDRESS_COLUMN, 
				 G_TYPE_STRING,        /* street        */
				 G_TYPE_STRING,        /* street number */
				 G_TYPE_STRING,        /* city          */
				 G_TYPE_STRING,        /* address type  */
				 G_TYPE_STRING,        /* zip           */ 
				 G_TYPE_STRING,        /* province      */
				 G_TYPE_STRING,        /* state         */
				 G_TYPE_STRING);       /* country       */
      break;

      /* net model */
    case NET_MODEL:
      model = gtk_list_store_new(LAST_NET_COLUMN, 
				 G_TYPE_STRING,        /* url       */
				 G_TYPE_STRING);       /* url type  */
      break;

      /* telephone model */
    case TELEPHONE_MODEL:
      model = gtk_list_store_new(LAST_TEL_COLUMN, 
				 G_TYPE_STRING,        /* telephone number */
				 G_TYPE_STRING);       /* telephone type   */
      break;
      
    default:
      break;
    }

  return GTK_TREE_MODEL(model);
}



void 
rubrica_model_populate_groups_view(GtkTreeView* view, RubricaGroupSet* set)
{
  GtkTreeModel* model;
  GtkTreeIter iter;  
  RubricaGroup* group;
  
  model = gtk_tree_view_get_model(view);

  rubrica_group_set_reset(set);
  group = rubrica_group_set_get_group(set);
  for(; group; group = rubrica_group_set_get_next_group(set))
    {
      gboolean enabled = FALSE;
      gchar* label = NULL;
      gchar* owner = NULL;
      gchar* pixmap = NULL;
      GdkPixbuf* pixbuf;
      
      g_object_get(group, "group-name", &label, "group-owner", &owner, 
		   "group-pixmap", &pixmap, "enabled", &enabled, NULL);
      
      if (g_file_test (pixmap, G_FILE_TEST_EXISTS))
	pixbuf = gdk_pixbuf_new_from_file_at_size (pixmap, 22, 22, NULL);
      else
	pixbuf = rubrica_utils_pixbuf_new(pixmap);
      
      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
			 GROUP_BELONG_COLUMN,   enabled, 
			 GROUP_PIXMAP_COLUMN,   pixbuf, 
			 GROUP_LABEL_COLUMN,    label,
			 GROUP_OWNER_COLUMN,    owner,
			 GROUP_PIXMAP_PATH,     pixmap,
			 -1);
    }
}


void 
rubrica_model_redisplay_groups_view(GtkTreeView* view, RubricaGroupSet* set)
{
  GtkTreeModel* model;
  GtkTreeIter iter;  
  RubricaGroup* group;
  
  model = gtk_tree_view_get_model(view);
  gtk_list_store_clear (GTK_LIST_STORE(model));

  rubrica_group_set_reset(set);
  group = rubrica_group_set_get_group(set);
  for(; group; group = rubrica_group_set_get_next_group(set))
    {
      gboolean enabled = FALSE;
      gchar* label = NULL;
      gchar* owner = NULL;
      gchar* pixmap = NULL;
      GdkPixbuf* pixbuf;
      
      g_object_get(group, "group-name", &label, "group-owner", &owner, 
		   "group-pixmap", &pixmap, "enabled", &enabled, NULL);  
      
      if (g_file_test (pixmap, G_FILE_TEST_EXISTS))
	pixbuf = gdk_pixbuf_new_from_file_at_size (pixmap, 22, 22, NULL);
      else
	pixbuf = rubrica_utils_pixbuf_new(pixmap);
      
      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
			 GROUP_BELONG_COLUMN,   enabled, 
			 GROUP_PIXMAP_COLUMN,   pixbuf, 
			 GROUP_LABEL_COLUMN,    label,
			 GROUP_OWNER_COLUMN,    owner,
			 GROUP_PIXMAP_PATH,     pixmap,
			 -1);
    }
}


void 
rubrica_model_update_groups_view(GtkTreeView* view, RubricaGroupSet* set)
{
  GtkTreeModel* model;
  GtkTreeIter iter;  
  RubricaGroup* group;
  
  model = gtk_tree_view_get_model(view);

  rubrica_group_set_reset(set);
  group = rubrica_group_set_get_group(set);
  for(; group; group = rubrica_group_set_get_next_group(set))
    {
      gboolean enabled = FALSE;
      gchar* label = NULL;
      gchar* owner = NULL;
      gchar* pixmap = NULL;
      GdkPixbuf* pixbuf;
      
      g_object_get(group, "group-name", &label, "group-owner", &owner, 
		   "group-pixmap", &pixmap, "enabled", &enabled, NULL);
      
      if (g_file_test (pixmap, G_FILE_TEST_EXISTS))
	pixbuf = gdk_pixbuf_new_from_file_at_size (pixmap, 22, 22, NULL);
      else
	pixbuf = rubrica_utils_pixbuf_new(pixmap);
      
      gtk_list_store_append(GTK_LIST_STORE(model), &iter);
      gtk_list_store_set(GTK_LIST_STORE(model), &iter, 
			 GROUP_BELONG_COLUMN,   enabled, 
			 GROUP_PIXMAP_COLUMN,   pixbuf, 
			 GROUP_LABEL_COLUMN,    label,
			 GROUP_OWNER_COLUMN,    owner,
			 GROUP_PIXMAP_PATH,     pixmap,
			 -1);
    }
}


void 
rubrica_model_add_columns(GtkTreeView *view, RubricaColumnsType type)
{
  GtkTreeViewColumn* column;
  GtkCellRenderer *renderer;
  GtkTreeModel *model;
  //  GConfClient *client;    
  gint col;
  
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));

  switch(type)
    {  
    case ADDRESSBOOK_COLUMNS:      
      column = gtk_tree_view_column_new();

      renderer = gtk_cell_renderer_pixbuf_new();
      gtk_tree_view_column_pack_start(column, renderer, FALSE);
      gtk_tree_view_column_set_attributes (column, renderer, "pixbuf", 
					   ADDRESSBOOK_PIXMAP_COLUMN, NULL);
      
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_column_pack_start(column, renderer, TRUE);
      gtk_tree_view_column_set_attributes (column, renderer, "text", 
					   ADDRESSBOOK_FILENAME_COLUMN, NULL);
      
      gtk_tree_view_append_column (GTK_TREE_VIEW(view), column);
      break;

    case CARDS_COLUMNS:
      //      client = gconf_client_get_default(); 
      
      /*
      rubrica_card_set_column_attributes(client, view, 
					 RUBRICA_KEY_SHOW_CARD_COLUMN_CATEGORIES, 
					 col-1);
      */
      
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes(view, -1, _("Card"), 
							renderer,    
							"text", CARD_COLUMN, 
							NULL);
      column = gtk_tree_view_get_column(view, col-1);
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					      CARD_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view, 
					 RUBRICA_KEY_SHOW_CARD_COLUMN_CARD,
					 col-1);
      */
      
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes(view, -1, 
							_("First Name"),
							renderer, "text", 
							FIRST_NAME_COLUMN,
							NULL);
      column = gtk_tree_view_get_column(view, col-1);
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					      FIRST_NAME_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view, 
					 RUBRICA_KEY_SHOW_CARD_COLUMN_FIRST_NAME,
					 col-1);
      */
      
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes (view, -1, 
							 _("Last Name"),
							 renderer, "text", 
							 LAST_NAME_COLUMN,
							 NULL);
      column = gtk_tree_view_get_column(view, col-1);
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					      LAST_NAME_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view, 
					 RUBRICA_KEY_SHOW_CARD_COLUMN_LAST_NAME, 
					 col-1);
      */
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes (view, -1,
							 _("Profession"),
							 renderer,  "text",
							 PROFESSION_COLUMN, 
							 NULL);
      column = gtk_tree_view_get_column(view, col-1);
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					      PROFESSION_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view, 
					 RUBRICA_KEY_SHOW_CARD_COLUMN_PROFESSION, 
					 col-1);
      */
      
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes (view, -1, _("City"), 
							 renderer,
							 "text", 
							 CITY_COLUMN, NULL);
      column = gtk_tree_view_get_column(view, col-1);
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					      CITY_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view, 
					 RUBRICA_KEY_SHOW_CARD_COLUMN_CITY, 
					 col-1);
      */
      
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes (view, -1, 
							 _("Country"), 
							 renderer, "text", 
							 COUNTRY_COLUMN,
							 NULL);
      column = gtk_tree_view_get_column(view, col-1);
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column),
					      COUNTRY_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view, 
					 RUBRICA_KEY_SHOW_CARD_COLUMN_COUNTRY, 
					 col-1);
      */
      
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Web"),
							 renderer, "text",
							 WEB_CARD_COLUMN,
							 NULL);
      column = gtk_tree_view_get_column(view, col-1);
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					      WEB_CARD_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view,
					 RUBRICA_KEY_SHOW_CARD_COLUMN_WEB, col-1);
      */
      
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes (view, -1, _("Email"),
							 renderer, "text",
							 EMAIL_CARD_COLUMN, 
							 NULL);
      column = gtk_tree_view_get_column(view, col-1);
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column), 
					      EMAIL_CARD_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view,
					 RUBRICA_KEY_SHOW_CARD_COLUMN_EMAIL, 
					 col-1);
      */
      
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes (view, -1, 
							 _("Telephone"),
							 renderer, "text", 
							 TELEPHONE_COLUMN,
							 NULL);
      column = gtk_tree_view_get_column(view, col-1);
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column),
					      TELEPHONE_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view, 
					 RUBRICA_KEY_SHOW_CARD_COLUMN_TELEPHONE, 
					 col-1);
      */
      
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes (view, -1, 
							 _("Company"), 
							 renderer, "text", 
							 COMPANY_COLUMN, NULL);
      column = gtk_tree_view_get_column(view, col-1);
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column),
					      COMPANY_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view, 
					 RUBRICA_KEY_SHOW_CARD_COLUMN_COMPANY, 
					 col-1);
      */
      
      renderer = gtk_cell_renderer_text_new();
      col = gtk_tree_view_insert_column_with_attributes (view, -1, 
							 _("Assigment"),
							 renderer, "text",
							 ASSIGMENT_COLUMN,
							 NULL);
      column = gtk_tree_view_get_column(view, col-1);     
      gtk_tree_view_column_set_sort_column_id(GTK_TREE_VIEW_COLUMN(column),
					      ASSIGMENT_COLUMN);
      /*
      rubrica_card_set_column_attributes(client, view, 
					 RUBRICA_KEY_SHOW_CARD_COLUMN_ASSIGMENT, 
					 col-1);             
      */
      
      
      gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), 
					   CARD_COLUMN, GTK_SORT_ASCENDING);
      
      /*
      gtk_tree_sortable_set_default_sort_func(GTK_TREE_SORTABLE(model), 
					      (GtkTreeIterCompareFunc) 
					      rubrica_sort_alfa,
					      GINT_TO_POINTER(CARD_COLUMN),
					      NULL);

      
      for (i= FIRST_NAME_COLUMN; i < RUBRICA_COLUMN; i++)
	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), i, 
					(GtkTreeIterCompareFunc) 
					rubrica_sort_alfa, 
					GINT_TO_POINTER(i), NULL);
      
      gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), RUBRICA_COLUMN,
				      (GtkTreeIterCompareFunc) 
				      rubrica_sort_date, 
				      GINT_TO_POINTER(RUBRICA_COLUMN), NULL);
      */
      break;
      
    case GROUP_COLUMNS:      
      renderer = gtk_cell_renderer_toggle_new();
      gtk_tree_view_insert_column_with_attributes(view, -1, "Belongs", 
						  renderer, "active", 
						  GROUP_BELONG_COLUMN,
						  NULL); 

      g_signal_connect(G_OBJECT(renderer), "toggled", 
		       G_CALLBACK(rubrica_model_toggle_group), model);  
      
      
      column = gtk_tree_view_column_new();
      gtk_tree_view_column_set_title(column, "Groups");

      renderer = gtk_cell_renderer_pixbuf_new();
      gtk_tree_view_column_pack_start(column, renderer, FALSE);
      gtk_tree_view_column_set_attributes (column, renderer, 
					   "pixbuf", GROUP_PIXMAP_COLUMN,
					   NULL);
      
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_column_pack_start(column, renderer, TRUE);
      gtk_tree_view_column_set_attributes (column, renderer, 
					   "text", GROUP_LABEL_COLUMN,
					   NULL);

      gtk_tree_view_append_column (GTK_TREE_VIEW(view), column);
      break;

    case APP_GROUP_COLUMNS:      
      column = gtk_tree_view_column_new();

      renderer = gtk_cell_renderer_pixbuf_new();
      gtk_tree_view_column_pack_start(column, renderer, FALSE);
      gtk_tree_view_column_set_attributes (column, renderer, 
					   "pixbuf", GROUP_PIXMAP_COLUMN,
					   NULL);
      
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_column_pack_start(column, renderer, TRUE);
      gtk_tree_view_column_set_attributes (column, renderer, 
					   "text", GROUP_LABEL_COLUMN,
					   NULL);

      gtk_tree_view_append_column (GTK_TREE_VIEW(view), column);
      break;

    case ADDRESS_COLUMNS:
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, /* tree view */
						   -1,  "Street",
						   renderer, "text",      
						   ADDRESS_STREET_COLUMN,  
						   NULL);

      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, 
						   -1,  "Number",
						   renderer, "text",      
						   ADDRESS_NUMBER_COLUMN,  
						   NULL);

      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, 
						   -1,  "City",
						   renderer, "text",      
						   ADDRESS_CITY_COLUMN,  
						   NULL);  

      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, 
						   -1,  "Address type",
						   renderer, "text",      
						   ADDRESS_TYPE_COLUMN,  
						   NULL);       
      break;

    case URI_COLUMNS:
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, 
						   -1,  "Net URL",
						   renderer, "text",      
						   URI_COLUMN,  
						   NULL);       
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, -1, "Type", 
						   renderer, "text", 
						   URI_TYPE_COLUMN, 
						   NULL);     
      break;

    case TELEPHONE_COLUMNS:
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, -1, "Number", 
						   renderer, "text", 
						   TELEPHONE_NUMBER_COLUMN, 
						   NULL);   
      
      renderer = gtk_cell_renderer_text_new();
      gtk_tree_view_insert_column_with_attributes (view, -1, "Type", 
						   renderer, "text", 
						   TELEPHONE_TYPE_COLUMN, 
						   NULL);     
      break;

    default:
      break;
    }
}

/*
void rubrica_model_set_column_attributes(GConfClient *client, 
					 GtkTreeView *view, 
					 gchar *key, gint col)
{
  GtkTreeViewColumn *column;
  
  column = gtk_tree_view_get_column(view, col);
*/
//  gtk_tree_view_column_set_resizable(column, TRUE);
//  gtk_tree_view_column_set_clickable(column, TRUE);
  /*
  gtk_tree_view_column_set_reorderable(column, TRUE);

  rubrica_preferences_column_config(client, column, key);
}
  */


void  
rubrica_model_toggle_group(GtkCellRendererToggle *cell,   
			   gchar *path_str, gpointer data)  
{ 
  GtkTreeModel *model;  
  GtkTreeIter iter;  
  GtkTreePath *path;  
  gboolean status;       

  model  = (GtkTreeModel *) data; 
  path   = gtk_tree_path_new_from_string(path_str);  

  gtk_tree_model_get_iter (model, &iter, path);  
  gtk_tree_model_get (model, &iter, GROUP_BELONG_COLUMN, &status, -1);  

  status ^= 1;  
  gtk_list_store_set(GTK_LIST_STORE(model), &iter,   
		     GROUP_BELONG_COLUMN, status, -1);  
}
